/*
 *  1541d64.h - 1541 emulation in .d64 file
 *
 *  Frodo (C) 1996 Christian Bauer
 */

#ifndef _1541D64_H
#define _1541D64_H

#include <stdio.h>
#include "Types.h"
#include "IEC.h"


// BAM structure
typedef struct {
	UBYTE	dir_track;		// Track...
	UBYTE	dir_sector;		// ...and sector of first directory block
	BYTE	fmt_type;		// Format type
	BYTE	pad0;
	UBYTE	bitmap[4*35];	// Sector allocation
	UBYTE	disk_name[18];	// Disk name
	UBYTE	id[2];			// Disk ID
	BYTE	pad1;
	UBYTE	fmt_char[2];	// Format characters
	BYTE	pad2[4];
	BYTE	pad3[85];
} BAM;

// Directory entry structure
typedef struct {
	UBYTE	type;			// File type
	UBYTE	track;			// Track...
	UBYTE	sector;			// ...and sector of first data block
	UBYTE	name[16];		// File name
	UBYTE	side_track;		// Track...
	UBYTE	side_sector;	// ...and sector of first side sector
	UBYTE	rec_len;		// Record length
	BYTE	pad0[4];
	UBYTE	ovr_track;		// Track...
	UBYTE	ovr_sector;		// ...and sector on overwrite
	UBYTE	num_blocks_l;	// Number of blocks, LSB
	UBYTE	num_blocks_h;	// Number of blocks, MSB
	BYTE	pad1[2];
} DirEntry;

// Directory block structure
typedef struct {
	UBYTE		next_track;
	UBYTE		next_sector;
	DirEntry	entry[8];
} Directory;


class D64Drive : public Drive {
public:
	D64Drive(IEC *iec, char *filepath);
	virtual ~D64Drive();
	virtual UBYTE Open(int channel, char *filename);
	virtual UBYTE Close(int channel);
	virtual UBYTE Read(int channel, UBYTE *byte);
	virtual UBYTE Write(int channel, UBYTE byte, CBOOL eoi);

private:
	UBYTE open_file(int channel, char *filename);
	void convert_filename(char *srcname, char *destname, int *filemode, int *filetype);
	CBOOL find_file(char *filename, int *track, int *sector);
	UBYTE open_file_ts(int channel, int track, int sector);
	UBYTE open_directory(char *pattern);
	UBYTE open_direct(int channel, char *filename);
	void close_all_channels();
	void execute_command(char *command);
	void block_read_cmd(char *command);
	void buffer_ptr_cmd(char *command);
	CBOOL parse_bcmd(char *cmd, int *arg1, int *arg2, int *arg3, int *arg4);
	int alloc_buffer(int want);
	void free_buffer(int buf);
	CBOOL read_sector(int track, int sector, UBYTE *buffer);
	int offset_from_ts(int track, int sector);

	FILE *the_file;			// File pointer for .d64 file

	UBYTE *ram;				// 2KB 1541 RAM
	BAM *bam;				// Pointer to BAM
	Directory dir;			// Buffer for directory blocks

	int chan_mode[16];		// Channel mode
	int chan_buf_num[16];	// Buffer number of channel (for direct access channels)
	UBYTE *chan_buf[16];	// Pointer to buffer
	UBYTE *buf_ptr[16];		// Pointer in buffer
	int buf_len[16];		// Remaining bytes in buffer

	CBOOL buf_free[4];		// Buffer 0..3 free?

	char cmd_buffer[44];	// Buffer for incoming command strings
	int cmd_len;			// Length of received command

	int image_header;		// Length of .d64 file header
};

#endif
