/*
 *  C64.h - Put the pieces together
 *
 *  Frodo (C) 1996 Christian Bauer
 */

#ifndef _C64_H
#define _C64_H

#include "Types.h"

#ifdef __BEOS__
#include <KernelKit.h>
#endif


class Prefs;
class C64Display;
class MOS6510;
class MOS6569;
class MOS6581;
class MOS6526_1;
class MOS6526_2;
class IEC;
class MOS6502_1541;
class Job1541;


class C64 {
public:
	C64();
	~C64();

	void Run(void);
	void Quit(void);
	void Pause(void);
	void Resume(void);
	void Reset(void);
	void NMI(void);
	void VBlank(CBOOL draw_frame);
	void NewPrefs(Prefs *prefs);

	UBYTE *RAM, *Basic, *Kernal,
		  *Char, *Color;		// C64
	UBYTE *RAM1541, *ROM1541;	// 1541

	C64Display *TheDisplay;

	MOS6510 *TheCPU;			// C64
	MOS6569 *TheVIC;
	MOS6581 *TheSID;
	MOS6526_1 *TheCIA1;
	MOS6526_2 *TheCIA2;
	IEC *TheIEC;

	MOS6502_1541 *TheCPU1541;	// 1541
	Job1541 *TheJob1541;

private:
	void c64_ctor(void);
	void c64_dtor(void);
	void open_close_joysticks(CBOOL oldjoy1, CBOOL oldjoy2, CBOOL newjoy1, CBOOL newjoy2);
	UBYTE poll_joystick(int fd);
	void patch_kernal(CBOOL fast_reset, CBOOL emul_1541_proc);

	CBOOL thread_running;	// Emulation thread is running
	CBOOL quit_thyself;		// Emulation thread shall quit
	CBOOL have_a_break;		// Emulation thread shall pause

	double start_time;

	int joy1fd, joy2fd;		// File descriptors for joysticks
	UBYTE joykey;			// Joystick keyboard emulation mask value

	UBYTE orig_kernal_1d84,	// Original contents of kernal locations $1d84 and $1d85
		  orig_kernal_1d85;	// (for undoing the Fast Reset patch)

#ifdef __BEOS__
	static long thread_invoc(void *obj);
	void thread_func(void);

	thread_id the_thread;
	sem_id pause_sem;
#endif
};


#endif
