/*
 *  CIA.h - 6526 emulation
 *
 *  Frodo (C) 1996 Christian Bauer
 */

#ifndef _CIA_H
#define _CIA_H

#include "Types.h"


class MOS6510;
class MOS6502_1541;
class MOS6569;
struct MOS6526State;


class MOS6526 {
public:
	void Reset(void);
	void GetState(MOS6526State *cs);
	void SetState(MOS6526State *cs);
	void EmulateLine(void);
	void CountTOD(void);
	virtual void TriggerInterrupt(int bit)=0;

protected:
	MOS6510 *the_cpu;	// Pointer to 6510

	UBYTE pra, prb, ddra, ddrb;

	UWORD ta, tb, latcha, latchb;

	UBYTE tod_10ths, tod_sec, tod_min, tod_hr;
	UBYTE alm_10ths, alm_sec, alm_min, alm_hr;

	UBYTE sdr, icr, cra, crb;
	UBYTE int_mask;

	int tod_divider;	// TOD frequency divider

	CBOOL tod_halt,		// Flag: TOD halted
		  ta_cnt_phi2,	// Flag: Timer A is counting Phi 2
		  tb_cnt_phi2,	// Flag: Timer B is counting Phi 2
		  tb_cnt_ta;	// Flag: Timer B is counting underflows of Timer A
};


class MOS6526_1 : public MOS6526 {
public:
	MOS6526_1(MOS6510 *CPU);

	void Reset(void);
	UBYTE ReadRegister(UWORD adr);
	void WriteRegister(UWORD adr, UBYTE byte);
	virtual void TriggerInterrupt(int bit);

	UBYTE KeyMatrix[8];	// C64 keyboard matrix, 1 bit/key (0: key down, 1: key up)
	UBYTE RevMatrix[8];	// Reversed keyboard matrix

	UBYTE Joystick1;	// Joystick 1 AND value
	UBYTE Joystick2;	// Joystick 2 AND value
};


class MOS6526_2 : public MOS6526{
public:
	MOS6526_2(MOS6510 *CPU, MOS6569 *VIC, MOS6502_1541 *CPU1541);

	void Reset(void);
	UBYTE ReadRegister(UWORD adr);
	void WriteRegister(UWORD adr, UBYTE byte);
	virtual void TriggerInterrupt(int bit);

	CBOOL DATAState;		// State of IEC lines
	CBOOL CLKState;
	CBOOL ATNState;

private:
	MOS6569 *the_vic;
	MOS6502_1541 *the_cpu_1541;
};


// CIA state
struct MOS6526State {
	UBYTE pra;
	UBYTE ddra;
	UBYTE prb;
	UBYTE ddrb;
	UBYTE ta_lo;
	UBYTE ta_hi;
	UBYTE tb_lo;
	UBYTE tb_hi;
	UBYTE tod_10ths;
	UBYTE tod_sec;
	UBYTE tod_min;
	UBYTE tod_hr;
	UBYTE sdr;
	UBYTE int_data;		// Pending interrupts
	UBYTE cra;
	UBYTE crb;
						// Additional registers
	UWORD latcha;		// Timer latches
	UWORD latchb;
	UBYTE alm_10ths;	// Alarm time
	UBYTE alm_sec;
	UBYTE alm_min;
	UBYTE alm_hr;
	UBYTE int_mask;		// Enabled interrupts
};

#endif
