/*
 *  Prefs.cpp - Global preferences
 *
 *  Frodo (C) 1996 Christian Bauer
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "Prefs.h"
#include "Display.h"
#include "main.h"


// These are the active preferences
Prefs ThePrefs;


/*
 *  Constructor: Set up preferences with defaults
 */

Prefs::Prefs()
{
	NormalCycles = 63;
	BadLineCycles = 23;
	CIACycles = 63;
	FloppyCycles = 64;
	SkipFrames = 2;

	for (int i=0; i<4; i++)
		DriveType[i] = DRVTYPE_DIR;

	strcpy(DrivePath[0], "64prgs");
	strcpy(DrivePath[1], "");
	strcpy(DrivePath[2], "");
	strcpy(DrivePath[3], "");

	SIDType = SIDTYPE_NONE;

	SpritesOn = TRUE;
	SpriteCollisions = TRUE;
	Joystick1On = FALSE;
	Joystick2On = FALSE;
	JoystickSwap = FALSE;
	LimitSpeed = FALSE;
	FastReset = FALSE;
	CIAIRQHack = FALSE;
	MapSlash = TRUE;
	Emul1541Proc = FALSE;
}


/*
 *  Check preferences for validity and correct if necessary
 */

void Prefs::Check(void)
{
	if (SkipFrames <= 0) SkipFrames = 1;

	if ((SIDType < SIDTYPE_NONE) || (SIDType > SIDTYPE_DIGITAL))
		SIDType = SIDTYPE_NONE;

	for (int i=0; i<4; i++)
		if ((DriveType[i] < DRVTYPE_DIR) || (DriveType[i] > DRVTYPE_D64))
			DriveType[i] = DRVTYPE_DIR;
}


/*
 *  Load preferences from file
 */

void Prefs::Load(char *filename)
{
	FILE *file;
	char line[256], keyword[256], value[256];

	if (file = fopen(filename, "r")) {
		while(fgets(line, 255, file)) {
			if (sscanf(line, "%s = %s\n", keyword, value) == 2) {
				if (!strcmp(keyword, "NormalCycles"))
					NormalCycles = atoi(value);
				else if (!strcmp(keyword, "BadLineCycles"))
					BadLineCycles = atoi(value);
				else if (!strcmp(keyword, "CIACycles"))
					CIACycles = atoi(value);
				else if (!strcmp(keyword, "FloppyCycles"))
					FloppyCycles = atoi(value);
				else if (!strcmp(keyword, "SkipFrames"))
					SkipFrames = atoi(value);
				else if (!strcmp(keyword, "DriveType8"))
					DriveType[0] = strcmp(value, "DIR") ? DRVTYPE_D64 : DRVTYPE_DIR;
				else if (!strcmp(keyword, "DriveType9"))
					DriveType[1] = strcmp(value, "DIR") ? DRVTYPE_D64 : DRVTYPE_DIR;
				else if (!strcmp(keyword, "DriveType10"))
					DriveType[2] = strcmp(value, "DIR") ? DRVTYPE_D64 : DRVTYPE_DIR;
				else if (!strcmp(keyword, "DriveType11"))
					DriveType[3] = strcmp(value, "DIR") ? DRVTYPE_D64 : DRVTYPE_DIR;
				else if (!strcmp(keyword, "DrivePath8"))
					strcpy(DrivePath[0], value);
				else if (!strcmp(keyword, "DrivePath9"))
					strcpy(DrivePath[1], value);
				else if (!strcmp(keyword, "DrivePath10"))
					strcpy(DrivePath[2], value);
				else if (!strcmp(keyword, "DrivePath11"))
					strcpy(DrivePath[3], value);
				else if (!strcmp(keyword, "SIDType"))
					SIDType = strcmp(value, "DIGITAL") ? SIDTYPE_NONE : SIDTYPE_DIGITAL;
				else if (!strcmp(keyword, "SpritesOn"))
					SpritesOn = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "SpriteCollisions"))
					SpriteCollisions = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "Joystick1On"))
					Joystick1On = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "Joystick2On"))
					Joystick2On = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "JoystickSwap"))
					JoystickSwap = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "LimitSpeed"))
					LimitSpeed = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "FastReset"))
					FastReset = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "CIAIRQHack"))
					CIAIRQHack = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "MapSlash"))
					MapSlash = !strcmp(value, "TRUE");
				else if (!strcmp(keyword, "Emul1541Proc"))
					Emul1541Proc = !strcmp(value, "TRUE");
			}
		}
		fclose(file);
	}
	Check();
}


/*
 *  Save preferences to file
 *  TRUE: success, FALSE: error
 */

CBOOL Prefs::Save(char *filename)
{
	FILE *file;

	Check();
	if (file = fopen(filename, "w")) {
		fprintf(file, "NormalCycles = %d\n", NormalCycles);
		fprintf(file, "BadLineCycles = %d\n", BadLineCycles);
		fprintf(file, "CIACycles = %d\n", CIACycles);
		fprintf(file, "FloppyCycles = %d\n", FloppyCycles);
		fprintf(file, "SkipFrames = %d\n", SkipFrames);
		for (int i=0; i<4; i++) {
			fprintf(file, "DriveType%d = %s\n", i+8, DriveType[i] == DRVTYPE_DIR ? "DIR" : "D64");
			fprintf(file, "DrivePath%d = %s\n", i+8, DrivePath[i]);
		}
		fprintf(file, "SIDType = %s\n", SIDType == SIDTYPE_NONE ? "NONE" : "DIGITAL");
		fprintf(file, "SpritesOn = %s\n", SpritesOn ? "TRUE" : "FALSE");
		fprintf(file, "SpriteCollisions = %s\n", SpriteCollisions ? "TRUE" : "FALSE");
		fprintf(file, "Joystick1On = %s\n", Joystick1On ? "TRUE" : "FALSE");
		fprintf(file, "Joystick2On = %s\n", Joystick2On ? "TRUE" : "FALSE");
		fprintf(file, "JoystickSwap = %s\n", JoystickSwap ? "TRUE" : "FALSE");
		fprintf(file, "LimitSpeed = %s\n", LimitSpeed ? "TRUE" : "FALSE");
		fprintf(file, "FastReset = %s\n", FastReset ? "TRUE" : "FALSE");
		fprintf(file, "CIAIRQHack = %s\n", CIAIRQHack ? "TRUE" : "FALSE");
		fprintf(file, "MapSlash = %s\n", MapSlash ? "TRUE" : "FALSE");
		fprintf(file, "Emul1541Proc = %s\n", Emul1541Proc ? "TRUE" : "FALSE");
		fclose(file);
		return TRUE;
	}
	return FALSE;
}


#ifdef __BEOS__
#include "Prefs_Be.i"
#endif

#ifdef AMIGA
#include "Prefs_Amiga.i"
#endif

#ifdef __unix
#include "Prefs_x.i"
#endif
