/*
 *  Prefs.h - Global preferences
 *
 *  Frodo (C) 1996 Christian Bauer
 */

#ifndef _PREFS_H
#define _PREFS_H

#include "Types.h"


// Drive types
enum {
	DRVTYPE_DIR,	// 1541 emulation in host file system
	DRVTYPE_D64		// 1541 emulation in .d64 file
};


// SID types
enum {
	SIDTYPE_NONE,	// SID emulation off
	SIDTYPE_DIGITAL	// Digital SID emulation
};


// Preferences data
class Prefs {
public:
	Prefs();
	void ShowEditor(CBOOL startup, char *prefs_name);
	void Check(void);
	void Load(char *filename);
	CBOOL Save(char *filename);

	int NormalCycles;		// Available CPU cycles in normal raster lines
	int BadLineCycles;		// Available CPU cycles in Bad Lines
	int CIACycles;			// CIA timer ticks per raster line
	int FloppyCycles;		// Available 1541 CPU cycles per line
	int SkipFrames;			// Draw every n-th frame

	int DriveType[4];		// Type of drive 8..11

	char DrivePath[4][256];	// Path for drive 8..11

	int SIDType;			// SID emulation type

	CBOOL SpritesOn;		// Sprite display is on
	CBOOL SpriteCollisions;	// Sprite collision detection is on
	CBOOL Joystick1On;		// Joystick connected to port 1 of host
	CBOOL Joystick2On;		// Joystick connected to port 2 of host
	CBOOL JoystickSwap;		// Swap joysticks 1<->2
	CBOOL LimitSpeed;		// Limit speed to 100%
	CBOOL FastReset;		// Skip RAM test on reset
	CBOOL CIAIRQHack;		// Write to CIA ICR clears IRQ
	CBOOL MapSlash;			// Map '/' in C64 filenames
	CBOOL Emul1541Proc;		// Enable processor-level 1541 emulation
};


// These are the active preferences
extern Prefs ThePrefs;

#endif
