/*
 *  SID.h - 6581 emulation
 *
 *  Frodo (C) 1996 Christian Bauer
 */

#ifndef _SID_H
#define _SID_H

#include "Types.h"

struct MOS6581State;


class Prefs;
class SIDRenderer;

// Class for administrative functions
class MOS6581 {
public:
	MOS6581();
	~MOS6581();

	void Reset(void);
	UBYTE ReadRegister(UWORD adr);
	void WriteRegister(UWORD adr, UBYTE byte);
	void NewPrefs(Prefs *prefs);
	void PauseSound(void);
	void ResumeSound(void);
	void GetState(MOS6581State *ss);
	void SetState(MOS6581State *ss);

private:
	void open_close_renderer(int old_type, int new_type);

	SIDRenderer *the_renderer;	// Pointer to current renderer
	UBYTE regs[25];				// Copies of the 25 write-only SID registers
};


// Renderers do the actual audio data processing
class SIDRenderer {
public:
	virtual ~SIDRenderer() {}
	virtual void Reset(void)=0;
	virtual void WriteRegister(UWORD adr, UBYTE byte)=0;
	virtual void Pause(void)=0;
	virtual void Resume(void)=0;
};


// SID state
struct MOS6581State {
	UBYTE freq_lo_1;
	UBYTE freq_hi_1;
	UBYTE pw_lo_1;
	UBYTE pw_hi_1;
	UBYTE ctrl_1;
	UBYTE AD_1;
	UBYTE SR_1;

	UBYTE freq_lo_2;
	UBYTE freq_hi_2;
	UBYTE pw_lo_2;
	UBYTE pw_hi_2;
	UBYTE ctrl_2;
	UBYTE AD_2;
	UBYTE SR_2;

	UBYTE freq_lo_3;
	UBYTE freq_hi_3;
	UBYTE pw_lo_3;
	UBYTE pw_hi_3;
	UBYTE ctrl_3;
	UBYTE AD_3;
	UBYTE SR_3;

	UBYTE fc_lo;
	UBYTE fc_hi;
	UBYTE res_filt;
	UBYTE mode_vol;

	UBYTE pot_x;
	UBYTE pot_y;
	UBYTE osc_3;
	UBYTE env_3;
};

#endif
