/*
 *  VIC.h - 6569R5 emulation (line based)
 *
 *  Frodo (C) 1996 Christian Bauer
 */

#ifndef _VIC_H
#define _VIC_H

#include "Types.h"

// Define this if you want global variables instead of member variables
#define GLOBAL_VARS


class MOS6510;
class C64Display;
class C64;
struct MOS6569State;


class MOS6569 {
public:
	MOS6569(C64 *c64, C64Display *disp, MOS6510 *CPU, UBYTE *RAM, UBYTE *Char, UBYTE *Color);

	UBYTE ReadRegister(UWORD adr);
	void WriteRegister(UWORD adr, UBYTE byte);
	int EmulateLine(void);
	void ChangedVA(UWORD new_va); // CIA VA14/15 has changed
	void GetState(MOS6569State *vd);

private:
#ifndef GLOBAL_VARS
	void vblank(void);
	void raster_irq(void);
	UBYTE *get_physical(UWORD adr);
	void make_mc_table(void);
	void el_std_text(UBYTE *p, UBYTE *q, UBYTE *r);
	void el_mc_text(UBYTE *p, UBYTE *q, UBYTE *r);
	void el_std_bitmap(UBYTE *p, UBYTE *q, UBYTE *r);
	void el_mc_bitmap(UBYTE *p, UBYTE *q, UBYTE *r);
	void el_ecm_text(UBYTE *p, UBYTE *q, UBYTE *r);
	void el_std_idle(UBYTE *p, UBYTE *r);
	void el_sprites(void);

	UWORD mx[8];				// VIC registers
	UBYTE mx8;
	UBYTE my[8];
	UBYTE ctrl1, ctrl2;
	UBYTE me, mxe, mye, mdp, mmc;
	UBYTE vbase;
	UBYTE irq_flag, irq_mask;
	UBYTE clx_spr, clx_bgr;
	UBYTE ec, b0c, b1c, b2c, b3c, mm0, mm1;
	UBYTE sc[8];

	UBYTE *ram, *char_rom, *color_ram; // Pointers to RAM and ROM
	C64 *the_c64;				// Pointer to C64
	C64Display *the_display;	// Pointer to C64Display
	MOS6510 *the_cpu;			// Pointer to 6510

	UBYTE colors[256];			// Indices of the 16 C64 colors (16 times mirrored to avoid "& 0x0f")

	UBYTE ec_color, b0c_color, b1c_color,
	      b2c_color, b3c_color;	// Indices for exterior/background colors
	UBYTE mm0_color, mm1_color;	// Indices for MOB multicolors
	UBYTE spr_color[8];			// Indices for MOB colors

	UWORD mc_color_lookup[4];

	UBYTE matrix_line[40];		// Buffer for video line, read in Bad Lines
	UBYTE color_line[40];		// Buffer for color line, read in Bad Lines

	UBYTE *chunky_ptr;			// Pointer in chunky bitmap buffer
	int xmod;					// Number of bytes per row

	UBYTE *matrix_base;			// Video matrix base
	UBYTE *char_base;			// Character generator base
	UBYTE *bitmap_base;			// Bitmap base

	UWORD raster_y;				// Current raster line
	UWORD irq_raster;			// Interrupt raster line
	UWORD dx_start;				// Comparison values for border logic
	UWORD dx_stop;
	UWORD dy_start;
	UWORD dy_stop;
	UWORD rc;					// Row counter
	UWORD vc;					// Video counter
	UWORD vcbase;				// Video counter base
	UWORD x_scroll;				// X scroll value
	UWORD y_scroll;				// Y scroll value
	UWORD cia_vabase;			// CIA VA14/15 video base

	int display_idx;			// Index of current display mode
	int skip_counter;			// Counter for frame-skipping

	UWORD mc[8];				// Sprite data counters
	UBYTE sprite_on;			// 8 Flags: Sprite display/DMA active

	UBYTE spr_coll_buf[0x180];	// Buffer for sprite-sprite collisions and priorities
	UBYTE fore_mask_buf[0x180/8];	// Foreground mask for sprite-graphics collisions and priorities
#ifndef __BEOS__
	UBYTE text_chunky_buf[40*8];	// Line graphics buffer
#endif

	CBOOL display_state;		// TRUE: Display state, FALSE: Idle state
	CBOOL border_on;			// Flag: Upper/lower border on
	CBOOL frame_skipped;		// Flag: Frame is being skipped
	CBOOL bad_lines_enabled;	// Flag: Bad Lines enabled for this frame
#endif
};


// VIC state
struct MOS6569State {
	UBYTE m0x;				// Sprite coordinates
	UBYTE m0y;
	UBYTE m1x;
	UBYTE m1y;
	UBYTE m2x;
	UBYTE m2y;
	UBYTE m3x;
	UBYTE m3y;
	UBYTE m4x;
	UBYTE m4y;
	UBYTE m5x;
	UBYTE m5y;
	UBYTE m6x;
	UBYTE m6y;
	UBYTE m7x;
	UBYTE m7y;
	UBYTE mx8;

	UBYTE ctrl1;			// Control registers
	UBYTE raster;
	UBYTE lpx;
	UBYTE lpy;
	UBYTE me;
	UBYTE ctrl2;
	UBYTE mye;
	UBYTE vbase;
	UBYTE irq_flag;
	UBYTE irq_mask;
	UBYTE mdp;
	UBYTE mmc;
	UBYTE mxe;
	UBYTE mm;
	UBYTE md;

	UBYTE ec;				// Color registers
	UBYTE b0c;
	UBYTE b1c;
	UBYTE b2c;
	UBYTE b3c;
	UBYTE mm0;
	UBYTE mm1;
	UBYTE m0c;
	UBYTE m1c;
	UBYTE m2c;
	UBYTE m3c;
	UBYTE m4c;
	UBYTE m5c;
	UBYTE m6c;
	UBYTE m7c;
							// Additional registers
	UBYTE pad0;
	UWORD irq_raster;		// IRQ raster line
	UWORD vc;				// Video counter
	UWORD vcbase;			// Video counter base
	UBYTE rc;				// Row counter
	UBYTE spr_dma;			// 8 Flags: Sprite display/DMA active
	UBYTE mc[8];			// Sprite data counters
	CBOOL display_state;		// TRUE: Display state, FALSE: Idle state
	CBOOL bad_line;			// Flag: Bad Line state
	CBOOL bad_line_enable;	// Flag: Bad Lines enabled for this frame

	UWORD bank_base;		// VIC bank base address
	UWORD matrix_base;		// Video matrix base
	UWORD char_base;		// Character generator base
	UWORD bitmap_base;		// Bitmap base
	UWORD sprite_base[8];	// Sprite bases
};

#endif
