/*
 *  main.h - Main program
 *
 *  Frodo (C) 1996 Christian Bauer
 */

#ifndef _MAIN_H
#define _MAIN_H

#include "Types.h"
#include "C64.h"



// Global variables
extern char AppDirPath[256];	// Path of application directory


/**
 **  BeOS specific stuff
 **/

#ifdef __BEOS__
#include <AppKit.h>

// Global variables
extern CBOOL FromShell;		// TRUE: Started from shell, SAM can be used


// Message codes
const ulong MSG_STARTUP = 'strt';	// Start emulation
const ulong MSG_PREFS = 'pref';		// Show preferences editor
const ulong MSG_PREFS_DONE = 'pdon'; // Preferences editor closed
const ulong MSG_RESET = 'rset';		// Reset C64
const ulong MSG_NMI = 'nmi ';		// Raise NMI
const ulong MSG_SAM = 'sam ';		// Invoke SAM


// Application signature
const ulong APP_SIGNATURE = 'Frod';


// Application class
class Frodo : public BApplication {
public:
	Frodo();
	virtual void ArgvReceived(int argc, char **argv);
	virtual void RefsReceived(BMessage *message);
	virtual void ReadyToRun(void);
	virtual void MessageReceived(BMessage *msg);
	virtual bool QuitRequested(void);
	virtual void MenusWillShow(void);
	virtual void AboutRequested(void);

	C64 *TheC64;

private:
	CBOOL load_rom_files(void);

	char prefs_path[256];	// Pathname of current preferences file
	CBOOL prefs_showing;		// TRUE: Preferences editor is on screen
};

#endif


/**
 **  AmigaOS specific stuff
 **/

#ifdef AMIGA

class Frodo {
public:
	Frodo();

	C64 *TheC64;

private:
	CBOOL load_rom_files(void);

	char prefs_path[256];	// Pathname of current preferences file
	CBOOL prefs_showing;		// TRUE: Preferences editor is on screen
	CBOOL from_shell;		// TRUE: Started from shell, SAM can be used
};

#endif


/**
 **  X specific stuff
 **/

#ifdef __unix

class Frodo {
public:
       Frodo();
       
       void Run(void);
       C64 *TheC64;

private:
       CBOOL load_rom_files(void);
};

#endif


#endif
